**********************************************************************
*
* CRC_V41 -- Encodes and decodes data blocks using CCITT V.41 CRC.
* ====================================================================
*
* David Evans, September 1999.                 Version No:`LOST_COUNT'  
*
* This software subroutine encodes and decodes data frames using a
* CRC that conforms to the CCITT V.41 recommendation. The software has
* been written in a manner that allows it to be easily and quickly
* tailored to your 68HC11 system.  A comprehensive guide to using this
* software can be found at the web-site this software was downloaded
* from.
*
* Prior to installing this subroutine, ensure that the <ORG $E000>
* command on line 59 is removed.
*
* This software has been rigorously tested.  However, although all
* efforts have been made to ensure that this software contains no bugs
* or errors, the author DOES NOT assume liability arising out of the
* application or use of any of the information presented herein.
*
* The author appreciates feedback -- Please feel free to forward any
* complaints, suggestions, corrections or comments to:
*
* arkaydia@hotmail.com
*
**********************************************************************


CRC_DIR         EQU     $07
CRC_RET         EQU     $08
TEMPOR1         EQU     $09
XOR4RX1         EQU     $20
XOR4RX2         EQU     $21
XOR4RX3         EQU     $22
XOR4TX1         EQU     $23
XOR4TX2         EQU     $24
XOR4TX3         EQU     $25
TX_DIVIDEND1    EQU     $33
TX_DIVIDEND2    EQU     $34
TX_DIVIDEND3    EQU     $35
RX_DIVIDEND1    EQU     $36
RX_DIVIDEND2    EQU     $37
RX_DIVIDEND3    EQU     $38
IN_FOR_COUNT    EQU     $29
OUT_FOR_COUNT   EQU     $31
GEN_POLY1       EQU     $88
GEN_POLY2       EQU     $10
GEN_POLY3       EQU     $80
FINAL_CHECK     EQU     $30
TX_BUFF         EQU     $50
RX_BUFF         EQU     $B0
OUTER_COUNT     EQU     31
CHKSUM1         EQU     32
CHKSUM2         EQU     33


        ORG     $E000

crc_V41 PSHA                    ;
        PSHB                    ;
        PSHX                    ;
        PSHY                    ;

        LDAA    CRC_DIR         ; Fetch direction flag.
        CMPA    #$01            ; If request is decode,
        BEQ     CRC_DEC         ; jump to decoding algorithm.
        LDAA    CRC_DIR         ; Fetch direction flag again.
        CMPA    #$00            ; If request is encode,
        BEQ     JMP_ENK         ; jump to encoding algorithm.
        LDAA    #$FF            ;
        STAA    CRC_RET         ; Otherwise return with failure.
        PULY                    ;
        PULX                    ;
        PULB                    ;
        PULA                    ;
        RTS                     ;
JMP_ENK JMP     CRC_ENC         ; Jump to encoding algorithm.


***********************************************************************
***********************************************************************
*                                                               
* Received data frames are DECODED using in this subroutine using CCITT
* V.41 Cyclic Redundancy Code recommendation.
*                                                               
***********************************************************************
***********************************************************************
CRC_DEC LDX     #RX_BUFF        ; Load RX buffer pointer.
        LDAA    0,X             ; Place 1st three bytes in DIVIDEND...
        STAA    RX_DIVIDEND1    ;
        STAA    XOR4RX1         ;
        LDAA    $01,X           ;
        STAA    RX_DIVIDEND2    ;
        STAA    XOR4RX2         ;
        LDAA    $02,X           ;
        STAA    RX_DIVIDEND3    ;
        ANDA    #$80            ; Remove all but the MSB.
        STAA    XOR4RX3         ;
        LDAA    #OUTER_COUNT    ; Initialise counter for the 'outer'
        STAA    OUT_FOR_COUNT   ; for loop.
        LDAA    #$08            ; Initialise counter for the 'inner'
        STAA    IN_FOR_COUNT    ; for loop.
        INX                     ;
        INX                     ;
                                
OUT_4   LDAA    #$00            ;
        STAA    $FINAL_CHECK    ;
RX_DIV  LDAA    #GEN_POLY1      ; Load accA with highest byte of G(x).
        CMPA    XOR4RX1         ; Compare with highest XRes byte,
        BLO     A000            ; & if G(x)<Dn, then XOR with G(x).
        LDAA    #GEN_POLY1      ;
        CMPA    XOR4RX1         ; Checking if G(x) > D(x).
        BHI     A005            ;
        LDAA    #GEN_POLY2      ;
        CMPA    XOR4RX2         ;
        BLO     A000            ;
        LDAA    #GEN_POLY2      ;
        CMPA    XOR4RX2         ;
        BHI     A005            ; Branch if G(x) < D(x).
        LDAA    #GEN_POLY3      ;
        CMPA    XOR4RX3         ;
        BHI     A005            ;
        BRA     A000            ; Branch if G(x) == D(x).
A005    JMP     XOR_0           ; Else, G(x)>Dn, so XOR with zero.
A000    JMP     XOR_GX          ;

NTRY2   LSL     RX_DIVIDEND1    ; NOW SHIFT THE DIVIDEND 1 BIT LEFT....
        LDAA    RX_DIVIDEND2    ; 
        ANDA    #$80            ;
        LSRA                    ; Most significant bit is now
        LSRA                    ; shifted sown to least significant.
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        STAA    TEMPOR1         ; Store in memory so it can be OR'd.
        LDAB    RX_DIVIDEND1    ;
        ORAB    TEMPOR1         ;
        STAB    RX_DIVIDEND1    ;
        LSL     RX_DIVIDEND2    ;
        LDAA    RX_DIVIDEND3    ;
        ANDA    #$80            ;
        LSRA                    ; Most significant bit is now
        LSRA                    ; shifted sown to least significant.
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        STAA    TEMPOR1         ; Store in memory so it can be OR'd.
        LDAB    RX_DIVIDEND2    ;
        ORAB    TEMPOR1         ;
        STAB    RX_DIVIDEND2    ; 
        LSL     RX_DIVIDEND3    ; 


* Now shift the XOR result left 1 bit...
*---------------------------------------
        LSL     XOR4RX1         ; Shift the first byte to 1 bit left.
        LDAA    XOR4RX1         ; Put shifted byte in accA,
        LDAB    XOR4RX2         ; & put the next byte in accB.
        LSRB                    ;
        LSRB                    ; The next several lines shift the 
        LSRB                    ; most significant bit down to the
        LSRB                    ; least significant bit.
        LSRB                    ;
        LSRB                    ;
        LSRB                    ;
        STAB    TEMPOR1         ; Generic buffer.
        ORAA    TEMPOR1         ; Concatenate LSB to accA.
        STAA    XOR4RX1         ; Put new byte in 1st Xres space.
        LSL     XOR4RX2         ; Shift 2nd XRes byte left 1 bit,
        LDAA    XOR4RX2         ; & put it in accA.
        LDAB    XOR4RX3         ; Put last XRes byte in accB,
        LSRB                    ; 
        LSRB                    ; The next several lines shift the
        LSRB                    ; most significant bit down to the
        LSRB                    ; least significant bit.
        LSRB                    ;
        LSRB                    ;
        LSRB                    ;
        STAB    TEMPOR1         ;
        ORAA    TEMPOR1         ; Concatenate LSB to accA.
        STAA    XOR4RX2         ; Put the new byte in its place.
        LSL     XOR4RX3         ; 



* Now drag the next bit down for the next division...
*----------------------------------------------------
        LDAB    RX_DIVIDEND3    ; Load accA with 3rd quotient byte
        ANDB    #$80            ; & keep only the MSB...
        STAB    XOR4RX3         ; Then store it away.
        DEC     IN_FOR_COUNT    ; Executed inner loop 8 times?
        BNE     JMP2RX          ; Keep looping if not.
        DEC     OUT_FOR_COUNT   ;
        BNE     JMP_OUT         ; Keep looping.
        JMP     A004            ; Jump to the next step if finished.
JMP2RX  JMP     RX_DIV          ; 

JMP_OUT INX                     ;
        LDAA    #$08            ; Reinitialise counter of the inner
        STAA    IN_FOR_COUNT    ; for loop.
        LDAA    0,X             ; Fetch next byte from data frame.
        STAA    RX_DIVIDEND3    ; 
        ANDA    #$80            ; 
        STAA    XOR4RX3         ;
        JMP     OUT_4           ;
                                 

* G1(x) was < D1 so XOR data with G(x)...
*========================================

XOR_GX  LDAA    #GEN_POLY1      ;
        EORA    XOR4RX1         ; XOR 1st XRes byte with 1st G(X) byte.
        STAA    XOR4RX1         ; Store it in XRes buffer.
        LDAA    #GEN_POLY2      ; Load accA with next G(x) byte,
        EORA    XOR4RX2         ; & XOR it with the 2nd XRes byte.
        STAA    XOR4RX2         ; Store result in XRes buffer.
        LDAA    #GEN_POLY3      ; Load accA with last bit of G(x),
        LDAB    XOR4RX3         ; Load accB with 3rd XRes byte,
        STAB    TEMPOR1         ; & store in a generic buffer.
        EORA    TEMPOR1         ; XOR 3rd data byte & last bit of G(x).
        ANDA    #$80            ; Get rid of all but the MSB,
        STAA    XOR4RX3         ; & store it away.
        JMP     NTRY2           ; Begin the next long div. step...

* G1(x) was > D1 so XOR data with zero...
*========================================

XOR_0   LDAA    #$00            ; 
        EORA    XOR4RX1         ; XOR 1st XRes byte with zero.
        STAA    XOR4RX1         ; Store it in XRes buffer.
        LDAA    #$00            ; Clear accA,
        EORA    XOR4RX2         ; & XOR it with the 2nd XRes byte.
        STAA    XOR4RX2         ; Store result in XRes buffer.
        LDAA    #$00            ; Clear accA.
        LDAB    XOR4RX3         ; Load accB with 3rd XRes byte,
        STAB    TEMPOR1         ; & store in a generic buffer.
        EORA    TEMPOR1         ; XOR 3rd XRes byte & last bit of G(x).
        ANDA    #$80            ; Get rid of all but the MSB,
        STAA    XOR4RX3         ; & store it away.
        JMP     NTRY2           ; 




A004    INX                     ;
        LDAA    0,X             ;
        STAA    RX_DIVIDEND3    ; 
        ANDA    #$80            ;   
        STAA    XOR4RX3         ;   
        LDAA    #$07            ; Do this seven times...
        STAA    IN_FOR_COUNT    ;

FINI    LDAA    #GEN_POLY1      ; Load accA with highest byte of G(x).
        CMPA    XOR4RX1         ; Compare with highest XRes byte,
        BLO     A007            ; & if G(x)<Dn, then XOR with G(x).
        LDAA    #GEN_POLY1      ;
        CMPA    XOR4RX1         ; Checking if G(x) > D(x).
        BHI     A006            ;
        LDAA    #GEN_POLY2      ;
        CMPA    XOR4RX2         ;
        BLO     A007            ;
        LDAA    #GEN_POLY2      ;
        CMPA    XOR4RX2         ;
        BHI     A006            ; Branch if G(x) < D(x).
        LDAA    #GEN_POLY3      ;
        CMPA    XOR4RX3         ;
        BHI     A006            ;
        BRA     A007            ; Branch if G(x) == D(x).
A006    JMP     XOR_01          ; Else, G(x)>Dn, so XOR with zero.
A007    JMP     XOR_GX1


NTRY3   LSL     RX_DIVIDEND1    ; NOW SHIFT DIVIDEND 1 BIT LEFT....
        LDAA    RX_DIVIDEND2    ; 
        ANDA    #$80            ;
        LSRA                    ; Most significant bit is now
        LSRA                    ; shifted sown to least significant.
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        STAA    TEMPOR1         ; Store in memory so it can be OR'd.
        LDAB    RX_DIVIDEND1    ;
        ORAB    TEMPOR1         ;
        STAB    RX_DIVIDEND1    ;
        LSL     RX_DIVIDEND2    ;
        LDAA    RX_DIVIDEND3    ;
        ANDA    #$80            ;
        LSRA                    ; Most significant bit is now
        LSRA                    ; shifted sown to least significant.
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ; 
        LSRA                    ;
        STAA    TEMPOR1         ; Store in memory so it can be OR'd.
        LDAB    RX_DIVIDEND2    ;
        ORAB    TEMPOR1         ;
        STAB    RX_DIVIDEND2    ; 
        LSL     RX_DIVIDEND3    ; 


* Now shift the XOR result left 1 bit...
*---------------------------------------
        LSL     XOR4RX1         ; Shift the first byte to 1 bit left.
        LDAA    XOR4RX1         ; Put shifted byte in accA,
        LDAB    XOR4RX2         ; & put the next byte in accB.
        LSRB                    ;
        LSRB                    ; The next several lines shift the 
        LSRB                    ; most significant bit down to the
        LSRB                    ; least significant bit.
        LSRB                    ;
        LSRB                    ;
        LSRB                    ;
        STAB    TEMPOR1         ; Generic buffer.
        ORAA    TEMPOR1         ; Concatenate LSB to accA.
        STAA    XOR4RX1         ; Put new byte in 1st Xres space.
        LSL     XOR4RX2         ; Shift 2nd XRes byte left 1 bit,
        LDAA    XOR4RX2         ; & put it in accA.
        LDAB    XOR4RX3         ; Put last XRes byte in accB,
        LSRB                    ; 
        LSRB                    ; The next several lines shift the
        LSRB                    ; most significant bit down to the
        LSRB                    ; least significant bit.
        LSRB                    ;
        LSRB                    ;
        LSRB                    ;
        STAB    TEMPOR1         ;
        ORAA    TEMPOR1         ; Concatenate LSB to accA.
        STAA    XOR4RX2         ; Put the new byte in its place.
        LSL     XOR4RX3         ; 


        LDAB    RX_DIVIDEND3    ; Load accA with 3rd quotient byte
        ANDB    #$80            ; & keep only the MSB...
        STAB    XOR4RX3         ; Then store it away.
        DEC     IN_FOR_COUNT    ; Executed loop 7 times?
        BNE     JMP_FIN         ; Keep looping if not.
        BRA     FINI_OR         ;
JMP_FIN JMP     FINI

* One final exclusive OR before finishing...
*-------------------------------------------
FINI_OR LDAA    #$55            ;
        STAA    $FINAL_CHECK    ;
        LDAA    #GEN_POLY1      ; Load accA with highest byte of G(x).
        CMPA    XOR4RX1         ; Compare with highest XRes byte,
        BLO     A008            ; & if G(x)<Dn, then XOR with G(x).
        LDAA    #GEN_POLY1      ;
        CMPA    XOR4RX1         ; Checking if G(x) > D(x).
        BHI     A009            ;
        LDAA    #GEN_POLY2      ;
        CMPA    XOR4RX2         ;
        BLO     A008            ;
        LDAA    #GEN_POLY2      ;
        CMPA    XOR4RX2         ;
        BHI     A009            ; Branch if G(x) < D(x).
        LDAA    #GEN_POLY3      ;
        CMPA    XOR4RX3         ;
        BHI     A009            ;
        BRA     A008            ; Branch if G(x) == D(x).
A009    JMP     XOR_01          ; Else, G(x)>Dn, so XOR with zero.
A008    JMP     XOR_GX1


* Shift the final XOR result left 1 bit...
*---------------------------------------
NTRY4   LSL     XOR4RX1         ; Shift the first byte to 1 bit left.
        LDAA    XOR4RX1         ; Put shifted byte in accA,
        LDAB    XOR4RX2         ; & put the next byte in accB.
        LSRB                    ;
        LSRB                    ; The next several lines shift the 
        LSRB                    ; most significant bit down to the
        LSRB                    ; least significant bit.
        LSRB                    ;
        LSRB                    ;
        LSRB                    ;
        STAB    TEMPOR1         ; Generic buffer.
        ORAA    TEMPOR1         ; Concatenate LSB to accA.
        STAA    XOR4RX1         ; Put new byte in 1st Xres space.
        LSL     XOR4RX2         ; Shift 2nd XRes byte left 1 bit,
        LDAA    XOR4RX2         ; & put it in accA.
        LDAB    XOR4RX3         ; Put last XRes byte in accB,
        LSRB                    ; 
        LSRB                    ; The next several lines shift the
        LSRB                    ; most significant bit down to the
        LSRB                    ; least significant bit.
        LSRB                    ;
        LSRB                    ;
        LSRB                    ;
        STAB    TEMPOR1         ;
        ORAA    TEMPOR1         ; Concatenate LSB to accA.
        STAA    XOR4RX2         ; Put the new byte in its place.
        LSL     XOR4RX3         ; 

        LDAA    XOR4RX1         ; Check the remainder...
        CMPA    #$00            ;
        BNE     FAIL            ; Return with failure if it's not 0.
        LDAA    XOR4RX2         ;
        CMPA    #$00            ;
        BNE     FAIL            ;
        LDAA    #$00            ;
        STAA    CRC_DIR         ;
        PULY                    ;
        PULX                    ;
        PULB                    ;
        PULA                    ;
        RTS                     ; Otherwise return with success!
FAIL    LDAA    #$FF            ;
        STAA    CRC_DIR         ;
        PULY                    ;
        PULX                    ;
        PULB                    ;
        PULA                    ;
        RTS                     ;

* G1(x) was < D1 so XOR data with G(x)...
*========================================

XOR_GX1 LDAA    #GEN_POLY1      ;
        EORA    XOR4RX1         ; XOR 1st XRes byte with 1st G(X) byte.
        STAA    XOR4RX1         ; Store it in XRes buffer.
        LDAA    #GEN_POLY2      ; Load accA with next G(x) byte,
        EORA    XOR4RX2         ; & XOR it with the 2nd XRes byte.
        STAA    XOR4RX2         ; Store result in XRes buffer.
        LDAA    #GEN_POLY3      ; Load accA with last bit of G(x),
        LDAB    XOR4RX3         ; Load accB with 3rd XRes byte,
        STAB    TEMPOR1         ; & store in a generic buffer.
        EORA    TEMPOR1         ; XOR 3rd data byte & last bit of G(x).
        ANDA    #$80            ; Get rid of all but the MSB,
        STAA    XOR4RX3         ; & store it away.
        LDAA    $FINAL_CHECK    ;
        CMPA    #$55            ;
        BEQ     A002            ;
        JMP     NTRY3           ; Begin the next long div. step...
A002    JMP     NTRY4           ;

* G1(x) was > D1 so XOR data with zero...
*========================================

XOR_01  LDAA    #$00            ; Clear accA.
        EORA    XOR4RX1         ; XOR 1st XRes byte with zero.
        STAA    XOR4RX1         ; Store it in XRes buffer.
        LDAA    #$00            ; Clear accA,
        EORA    XOR4RX2         ; & XOR it with the 2nd XRes byte.
        STAA    XOR4RX2         ; Store result in XRes buffer.
        LDAA    #$00            ; Clear accA.
        LDAB    XOR4RX3         ; Load accB with 3rd XRes byte,
        STAB    TEMPOR1         ; & store in a generic buffer.
        EORA    TEMPOR1         ; XOR 3rd XRes byte & last bit of G(x).
        ANDA    #$80            ; Get rid of all but the MSB,
        STAA    XOR4RX3         ; & store it away.
        LDAA    $FINAL_CHECK    ;
        CMPA    #$55            ;
        BEQ     A003            ;
        JMP     NTRY3           ; 
A003    JMP     NTRY4





*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
***********************************************************************
***********************************************************************
*                                                               
* This routine ENCODES a block of data to be transmitted using the
* CCITT V41 Cyclic Redundancy Code.
*                                                               
***********************************************************************
***********************************************************************
*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
CRC_ENC LDX     #TX_BUFF        ; Load Tx buffer pointer.
        LDAA    #$00            ;
        STAA    CHKSUM1,X       ; Append 16 0's to the checksum area.
        STAA    CHKSUM2,X       ;
        LDAA    0,X             ; Place 1st three bytes in DIVIDEND...
        STAA    TX_DIVIDEND1    ;
        STAA    XOR4TX1         ;
        LDAA    $01,X           ;
        STAA    TX_DIVIDEND2    ;
        STAA    XOR4TX2         ;
        LDAA    $02,X           ;
        STAA    TX_DIVIDEND3    ;
        ANDA    #$80            ; Remove all but the MSB.
        STAA    XOR4TX3         ;
        LDAA    #OUTER_COUNT    ; Initialise counter for the 'outer'
        STAA    OUT_FOR_COUNT   ; for loop.
        LDAA    #$08            ; Initialise counter for the 'inner'
        STAA    IN_FOR_COUNT    ; for loop.
        INX                     ;
        INX                     ;
                                
OUT_FOR LDAA    #$00            ;
        STAA    $FINAL_CHECK    ;
TX_DIV  LDAA    #GEN_POLY1      ; Load accA with highest byte of G(x).
        CMPA    XOR4TX1         ; Compare with highest XRes byte,
        BLO     B000            ; & if G(x)<Dn, then XOR with G(x).
        LDAA    #GEN_POLY1      ;
        CMPA    XOR4TX1         ; Checking if G(x) > D(x).
        BHI     B005            ;
        LDAA    #GEN_POLY2      ;
        CMPA    XOR4TX2         ;
        BLO     B000            ;
        LDAA    #GEN_POLY2      ;
        CMPA    XOR4TX2         ;
        BHI     B005            ; Branch if G(x) < D(x).
        LDAA    #GEN_POLY3      ;
        CMPA    XOR4TX3         ;
        BHI     B005            ;
        BRA     B000            ; Branch if G(x) == D(x).
B005    JMP     XOR0            ; Else, G(x)>Dn, so XOR with zero.
B000    JMP     XORGX           ;

ENTRY2  LSL     TX_DIVIDEND1    ; NOW SHIFT DIVIDEND 1 BIT LEFT....
        LDAA    TX_DIVIDEND2    ; 
        ANDA    #$80            ;
        LSRA                    ; Most significant bit is now
        LSRA                    ; shifted sown to least significant.
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        STAA    TEMPOR1         ; Store in memory so it can be OR'd.
        LDAB    TX_DIVIDEND1    ;
        ORAB    TEMPOR1         ;
        STAB    TX_DIVIDEND1    ;
        LSL     TX_DIVIDEND2    ;
        LDAA    TX_DIVIDEND3    ;
        ANDA    #$80            ;
        LSRA                    ; Most significant bit is now
        LSRA                    ; shifted sown to least significant.
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        STAA    TEMPOR1         ; Store in memory so it can be OR'd.
        LDAB    TX_DIVIDEND2    ;
        ORAB    TEMPOR1         ;
        STAB    TX_DIVIDEND2    ; 
        LSL     TX_DIVIDEND3    ; 


* Now shift the XOR result left 1 bit...
*---------------------------------------
        LSL     XOR4TX1         ; Shift the first byte to 1 bit left.
        LDAA    XOR4TX1         ; Put shifted byte in accA,
        LDAB    XOR4TX2         ; & put the next byte in accB.
        LSRB                    ;
        LSRB                    ; The next several lines shift the 
        LSRB                    ; most significant bit down to the
        LSRB                    ; least significant bit.
        LSRB                    ;
        LSRB                    ;
        LSRB                    ;
        STAB    TEMPOR1         ; Generic buffer.
        ORAA    TEMPOR1         ; Concatenate LSB to accA.
        STAA    XOR4TX1         ; Put new byte in 1st Xres space.
        LSL     XOR4TX2         ; Shift 2nd XRes byte left 1 bit,
        LDAA    XOR4TX2         ; & put it in accA.
        LDAB    XOR4TX3         ; Put last XRes byte in accB,
        LSRB                    ; 
        LSRB                    ; The next several lines shift the
        LSRB                    ; most significant bit down to the
        LSRB                    ; least significant bit.
        LSRB                    ;
        LSRB                    ;
        LSRB                    ;
        STAB    TEMPOR1         ;
        ORAA    TEMPOR1         ; Concatenate LSB to accA.
        STAA    XOR4TX2         ; Put the new byte in its place.
        LSL     XOR4TX3         ; 



* Now drag the next bit down for the next division...
*----------------------------------------------------
        LDAB    TX_DIVIDEND3    ; Load accA with 3rd quotient byte
        ANDB    #$80            ; & keep only the MSB...
        STAB    XOR4TX3         ; Then store it away.
        DEC     IN_FOR_COUNT    ; Executed inner loop 8 times?
        BNE     JMP2TX          ; Keep looping if not.
        DEC     OUT_FOR_COUNT   ;
        BNE     JMP2OUT         ; Keep looping otherwise.
        JMP     B004            ; Jump to next step if finished.
JMP2TX  JMP     TX_DIV          ; 

JMP2OUT INX                     ;
        LDAA    #$08            ; Reinitialise counter of the inner
        STAA    IN_FOR_COUNT    ; for loop.
        LDAA    0,X             ; Fetch next byte from data frame.
        STAA    TX_DIVIDEND3    ; 
        ANDA    #$80            ; 
        STAA    XOR4TX3         ;
        JMP     OUT_FOR         ;
                                 

* G1(x) was < D1 so XOR data with G(x)...
*========================================

XORGX   LDAA    #GEN_POLY1      ;
        EORA    XOR4TX1         ; XOR 1st XRes byte with 1st G(X) byte.
        STAA    XOR4TX1         ; Store it in XRes buffer.
        LDAA    #GEN_POLY2      ; Load accA with next G(x) byte,
        EORA    XOR4TX2         ; & XOR it with the 2nd XRes byte.
        STAA    XOR4TX2         ; Store result in XRes buffer.
        LDAA    #GEN_POLY3      ; Load accA with last bit of G(x),
        LDAB    XOR4TX3         ; Load accB with 3rd XRes byte,
        STAB    TEMPOR1         ; & store in a generic buffer.
        EORA    TEMPOR1         ; XOR 3rd data byte & last bit of G(x).
        ANDA    #$80            ; Get rid of all but the MSB,
        STAA    XOR4TX3         ; & store it away.
        JMP     ENTRY2          ; Begin the next long div. step...

* G1(x) was > D1 so XOR data with zero...
*========================================

XOR0    LDAA    #$00            ; 
        EORA    XOR4TX1         ; XOR 1st XRes byte with zero.
        STAA    XOR4TX1         ; Store it in XRes buffer.
        LDAA    #$00            ; Clear accA,
        EORA    XOR4TX2         ; & XOR it with the 2nd XRes byte.
        STAA    XOR4TX2         ; Store result in XRes buffer.
        LDAA    #$00            ; Clear accA.
        LDAB    XOR4TX3         ; Load accB with 3rd XRes byte,
        STAB    TEMPOR1         ; & store in a generic buffer.
        EORA    TEMPOR1         ; XOR 3rd XRes byte & last bit of G(x).
        ANDA    #$80            ; Get rid of all but the MSB,
        STAA    XOR4TX3         ; & store it away.
        JMP     ENTRY2          ; 




B004    INX                     ;
        LDAA    0,X             ;
        STAA    TX_DIVIDEND3    ; 
        ANDA    #$80            ;   
        STAA    XOR4TX3         ;   
        LDAA    #$07            ; Do this seven times...
        STAA    IN_FOR_COUNT    ;

FIN     LDAA    #GEN_POLY1      ; Load accA with highest byte of G(x).
        CMPA    XOR4TX1         ; Compare with highest XRes byte,
        BLO     B007            ; & if G(x)<Dn, then XOR with G(x).
        LDAA    #GEN_POLY1      ;
        CMPA    XOR4TX1         ; Checking if G(x) > D(x).
        BHI     B006            ;
        LDAA    #GEN_POLY2      ;
        CMPA    XOR4TX2         ;
        BLO     B007            ;
        LDAA    #GEN_POLY2      ;
        CMPA    XOR4TX2         ;
        BHI     B006            ; Branch if G(x) < D(x).
        LDAA    #GEN_POLY3      ;
        CMPA    XOR4TX3         ;
        BHI     B006            ;
        BRA     B007            ; Branch if G(x) == D(x).
B006    JMP     XOR01           ; Else, G(x)>Dn, so XOR with zero.
B007    JMP     XORGX1          ;


ENTRY3  LSL     TX_DIVIDEND1    ; SHIFT THE DIVIDEND 1 BIT LEFT....
        LDAA    TX_DIVIDEND2    ; 
        ANDA    #$80            ;
        LSRA                    ; Most significant bit is now
        LSRA                    ; shifted sown to least significant.
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        STAA    TEMPOR1         ; Store in memory so it can be OR'd.
        LDAB    TX_DIVIDEND1    ;
        ORAB    TEMPOR1         ;
        STAB    TX_DIVIDEND1    ;
        LSL     TX_DIVIDEND2    ;
        LDAA    TX_DIVIDEND3    ;
        ANDA    #$80            ;
        LSRA                    ; Most significant bit is now
        LSRA                    ; shifted sown to least significant.
        LSRA                    ;
        LSRA                    ;
        LSRA                    ;
        LSRA                    ; 
        LSRA                    ;
        STAA    TEMPOR1         ; Store in memory so it can be OR'd.
        LDAB    TX_DIVIDEND2    ;
        ORAB    TEMPOR1         ;
        STAB    TX_DIVIDEND2    ; 
        LSL     TX_DIVIDEND3    ; 


* Now shift the XOR result left 1 bit...
*---------------------------------------
        LSL     XOR4TX1         ; Shift the first byte to 1 bit left.
        LDAA    XOR4TX1         ; Put shifted byte in accA,
        LDAB    XOR4TX2         ; & put the next byte in accB.
        LSRB                    ;
        LSRB                    ; The next several lines shift the 
        LSRB                    ; most significant bit down to the
        LSRB                    ; least significant bit.
        LSRB                    ;
        LSRB                    ;
        LSRB                    ;
        STAB    TEMPOR1         ; Generic buffer.
        ORAA    TEMPOR1         ; Concatenate LSB to accA.
        STAA    XOR4TX1         ; Put new byte in 1st Xres space.
        LSL     XOR4TX2         ; Shift 2nd XRes byte left 1 bit,
        LDAA    XOR4TX2         ; & put it in accA.
        LDAB    XOR4TX3         ; Put last XRes byte in accB,
        LSRB                    ; 
        LSRB                    ; The next several lines shift the
        LSRB                    ; most significant bit down to the
        LSRB                    ; least significant bit.
        LSRB                    ;
        LSRB                    ;
        LSRB                    ;
        STAB    TEMPOR1         ;
        ORAA    TEMPOR1         ; Concatenate LSB to accA.
        STAA    XOR4TX2         ; Put the new byte in its place.
        LSL     XOR4TX3         ; 


        LDAB    TX_DIVIDEND3    ; Load accA with 3rd quotient byte
        ANDB    #$80            ; & keep only the MSB...
        STAB    XOR4TX3         ; Then store it away.
        DEC     IN_FOR_COUNT    ; Executed loop 7 times?
        BNE     JMP2FIN         ; Keep looping if not.
        BRA     FIN_XOR         ;
JMP2FIN JMP     FIN

* One final exclusive OR before finishing...
*-------------------------------------------
FIN_XOR LDAA    #$55            ;
        STAA    $FINAL_CHECK    ;
        LDAA    #GEN_POLY1      ; Load accA with highest byte of G(x).
        CMPA    XOR4TX1         ; Compare with highest XRes byte,
        BLO     B008            ; & if G(x)<Dn, then XOR with G(x).
        LDAA    #GEN_POLY1      ;
        CMPA    XOR4TX1         ; Checking if G(x) > D(x).
        BHI     B009            ;
        LDAA    #GEN_POLY2      ;
        CMPA    XOR4TX2         ;
        BLO     B008            ;
        LDAA    #GEN_POLY2      ;
        CMPA    XOR4TX2         ;
        BHI     B009            ; Branch if G(x) < D(x).
        LDAA    #GEN_POLY3      ;
        CMPA    XOR4TX3         ;
        BHI     B009            ;
        BRA     B008            ; Branch if G(x) == D(x).
B009    JMP     XOR01           ; Else, G(x)>Dn, so XOR with zero.
B008    JMP     XORGX1


* Shift the final XOR result left 1 bit...
*---------------------------------------
ENTRY4  LSL     XOR4TX1         ; Shift the first byte to 1 bit left.
        LDAA    XOR4TX1         ; Put shifted byte in accA,
        LDAB    XOR4TX2         ; & put the next byte in accB.
        LSRB                    ;
        LSRB                    ; The next several lines shift the 
        LSRB                    ; most significant bit down to the
        LSRB                    ; least significant bit.
        LSRB                    ;
        LSRB                    ;
        LSRB                    ;
        STAB    TEMPOR1         ; Generic buffer.
        ORAA    TEMPOR1         ; Concatenate LSB to accA.
        STAA    XOR4TX1         ; Put new byte in 1st Xres space.
        LSL     XOR4TX2         ; Shift 2nd XRes byte left 1 bit,
        LDAA    XOR4TX2         ; & put it in accA.
        LDAB    XOR4TX3         ; Put last XRes byte in accB,
        LSRB                    ; 
        LSRB                    ; The next several lines shift the
        LSRB                    ; most significant bit down to the
        LSRB                    ; least significant bit.
        LSRB                    ;
        LSRB                    ;
        LSRB                    ;
        STAB    TEMPOR1         ;
        ORAA    TEMPOR1         ; Concatenate LSB to accA.
        STAA    XOR4TX2         ; Put the new byte in its place.
        LSL     XOR4TX3         ; 

        LDX     #TX_BUFF        ; Appending calculated checksum to 
        LDAA    XOR4TX1         ; the data frame to be transmitted.
        STAA    CHKSUM1,X       ;
        LDAA    XOR4TX2         ;
        STAA    CHKSUM2,X       ;
        LDAA    #$00            ;
        STAA    CRC_DIR         ;
        PULY                    ;
        PULX                    ;
        PULB                    ;
        PULA                    ;
        RTS                     ; Return from subroutine with success.


* G1(x) was < D1 so XOR data with G(x)...
*========================================

XORGX1  LDAA    #GEN_POLY1      ;
        EORA    XOR4TX1         ; XOR 1st XRes byte with 1st G(X) byte.
        STAA    XOR4TX1         ; Store it in XRes buffer.
        LDAA    #GEN_POLY2      ; Load accA with next G(x) byte,
        EORA    XOR4TX2         ; & XOR it with the 2nd XRes byte.
        STAA    XOR4TX2         ; Store result in XRes buffer.
        LDAA    #GEN_POLY3      ; Load accA with last bit of G(x),
        LDAB    XOR4TX3         ; Load accB with 3rd XRes byte,
        STAB    TEMPOR1         ; & store in a generic buffer.
        EORA    TEMPOR1         ; XOR 3rd data byte & last bit of G(x).
        ANDA    #$80            ; Get rid of all but the MSB,
        STAA    XOR4TX3         ; & store it away.
        LDAA    $FINAL_CHECK    ;
        CMPA    #$55            ;
        BEQ     B002            ;
        JMP     ENTRY3          ; Begin the next long div. step...
B002    JMP     ENTRY4

* G1(x) was > D1 so XOR data with zero...
*========================================

XOR01   LDAA    #$00            ; Clear accA.
        EORA    XOR4TX1         ; XOR 1st XRes byte with zero.
        STAA    XOR4TX1         ; Store it in XRes buffer.
        LDAA    #$00            ; Clear accA,
        EORA    XOR4TX2         ; & XOR it with the 2nd XRes byte.
        STAA    XOR4TX2         ; Store result in XRes buffer.
        LDAA    #$00            ; Clear accA.
        LDAB    XOR4TX3         ; Load accB with 3rd XRes byte,
        STAB    TEMPOR1         ; & store in a generic buffer.
        EORA    TEMPOR1         ; XOR 3rd XRes byte & last bit of G(x).
        ANDA    #$80            ; Get rid of all but the MSB,
        STAA    XOR4TX3         ; & store it away.
        LDAA    $FINAL_CHECK    ;
        CMPA    #$55            ;
        BEQ     B003            ;
        JMP     ENTRY3          ; 
B003    JMP     ENTRY4          ;




* Version No.`LOST_COUNT', late Saturday afternoon, September 18, 1999.
*
* Please forward any comments, suggestions, or corrections to:
* arkaydia@hotmail.com


